/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_SLOTITERATOR_HH
#define DUNE_FEM_SLOTITERATOR_HH

namespace Dune
{

  namespace Fem
  {

  namespace
  {

    template< class Container >
    struct ExtractElement
    {
      typedef typename Container :: ElementType Type;
    };

    template< class Container >
    struct ExtractElement< const Container >
    {
      typedef const typename Container :: ElementType Type;
    };

  }



  template< class StorageIterator >
  class SlotIterator
  {
    typedef SlotIterator< StorageIterator > ThisType;

  public:
    typedef StorageIterator StorageIteratorType;
    typedef typename StorageIteratorType :: ElementType ElementArrayType;
    typedef typename ExtractElement< ElementArrayType > :: Type ElementType;

  protected:
    StorageIteratorType storageIterator_;
    unsigned int slot_;

  public:
    inline SlotIterator ( const StorageIteratorType &storageIterator,
                          unsigned int slot )
    : storageIterator_( storageIterator ),
      slot_( slot )
    {}

    inline SlotIterator ( const ThisType &other )
    : storageIterator_( other.storageIterator_ ),
      slot_( other.slot_ )
    {}

    inline ThisType &operator= ( const ThisType &other )
    {
      storageIterator_ = other.storageIterator_;
      slot_ = other.slot_;
    }

    inline ThisType &operator++ ()
    {
      ++storageIterator_;
      return *this;
    }

    inline ElementType &operator* ()
    {
      return (*storageIterator_)[ slot_ ];
    }

    inline ElementType *operator-> ()
    {
      return &(operator*());
    }

    inline bool operator== ( const ThisType &other ) const
    {
      return
        (storageIterator_ == other.storageIterator_) && (slot_ == other.slot_);
    }

    inline bool operator!= ( const ThisType &other ) const
    {
      return !(*this == other);
    }
  };

  } // namespace Fem

} // namespace Dune
 
#endif
