/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_MANAGEDVECTORFUNCTION_HH
#define DUNE_FEM_MANAGEDVECTORFUNCTION_HH

#include <dune/fem/function/vectorfunction/vectorfunction.hh>
#include <dune/fem/space/common/dofmanager.hh>

namespace Dune
{
  namespace Fem 
  {

  template< class DiscreteFunctionSpace,
            class DofVector >
  class ManagedDiscreteFunction
    < VectorDiscreteFunction< DiscreteFunctionSpace, DofVector > >
  : public VectorDiscreteFunction< DiscreteFunctionSpace, DofVector >
  {
    typedef VectorDiscreteFunction< DiscreteFunctionSpace, DofVector > BaseType;
    typedef ManagedDiscreteFunction< BaseType > ThisType;

  public:
    typedef ThisType DiscreteFunctionType;

    typedef typename BaseType :: DiscreteFunctionSpaceType
      DiscreteFunctionSpaceType;
    typedef typename BaseType :: DofVectorType DofVectorType;

  protected:
    typedef typename DiscreteFunctionSpaceType :: GridPartType :: GridType
      GridType;

  protected:
    DofStorageInterface *memObject_;

  public:
    inline ManagedDiscreteFunction ( const std :: string &name,
                                     const DiscreteFunctionSpaceType &dfSpace )
    : BaseType( name, dfSpace, allocDofVector( name, dfSpace ) )
    {}

    inline explicit ManagedDiscreteFunction ( const BaseType &other )
    : BaseType( other.name(), other.space(),
                allocDofVector( other.name(), other.space() ) )
    {
      BaseType :: assign ( other );
    }

    inline ManagedDiscreteFunction ( const ThisType &other )
    : BaseType( other.name(), other.space(),
                allocDofVector( other.name(), other.space() ) )
    {
      BaseType :: assign ( other );
    }

    inline ~ManagedDiscreteFunction ()
    {
      if( memObject_ ) 
        delete memObject_ ;
    }

    inline void enableDofCompression ()
    {
      if( memObject_ )
        memObject_->enableDofCompression();
    }

  private:
    inline DofVectorType &
    allocDofVector ( const std :: string &name,
                     const DiscreteFunctionSpaceType &dfSpace )
    {
      // allocate managed dof storage 
      std :: pair< DofStorageInterface *, DofVectorType * > memPair
        = allocateManagedDofStorage( dfSpace.grid(), dfSpace.mapper(), 
                                     name , (DofVectorType *) 0);
      memObject_ = memPair.first;
      return *(memPair.second);
    }
  };

  } // namespace Fem

  // #if DUNE_FEM_COMPATIBILITY  
  // put this in next version 1.4 

  using Fem :: ManagedDiscreteFunction ;
  // #endif // DUNE_FEM_COMPATIBILITY

} // namespace Dune

#endif
