/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_IDGRIDPART_CAPABILITIES_HH
#define DUNE_FEM_GRIDPART_IDGRIDPART_CAPABILITIES_HH

//- dune-common includes
#include <dune/common/typetraits.hh>

//- dune-geometry includes
#include <dune/geometry/genericgeometry/topologytypes.hh>

//- dune-fem includes
#include <dune/fem/gridpart/common/capabilities.hh>
#include <dune/fem/misc/selectvalue.hh>


namespace Dune
{

  namespace Fem
  {

    // Forward declaration
    // -------------------

    template< class >
    class IdGridPart;



    namespace GridPartCapabilities
    {

      template< class HostGridPartType >
      struct hasGrid< IdGridPart< HostGridPartType > >
      {
        static const bool v = true; 
      };


      template< class HostGridPartType >
      struct hasSingleGeometryType< IdGridPart< HostGridPartType > >
      {
        static const bool v = hasSingleGeometryType< HostGridPartType >::v;
        static const unsigned int topologyId 
          = SelectUnsignedValue< v, hasSingleGeometryType< HostGridPartType >::topologyId, ~0 >::Value;
      };


      template< class HostGridPartType >
      struct isCartesian< IdGridPart< HostGridPartType > >
      {
        static const bool v = isCartesian< HostGridPartType >::v;
      };


      template< class HostGridPartType, int codim  >
      struct hasEntity< IdGridPart< HostGridPartType >, codim >
      {
        static const bool v = hasEntity< HostGridPartType, codim >::v; 
      };


      template< class HostGridPartType >
      struct isParallel< IdGridPart< HostGridPartType > >
      {
        static const bool v = isParallel< HostGridPartType >::v;
      };


      template< class HostGridPartType, int codim >
      struct canCommunicate< IdGridPart< HostGridPartType >, codim >
      {
        static const bool v = canCommunicate< HostGridPartType, codim >::v;
      };


      template< class HostGridPartType >
      struct isConforming< IdGridPart< HostGridPartType > >
      {
        static const bool v = isConforming< HostGridPartType >::v;
      };

    } // end namespace GridPartCapabilities

  } // end namespace Fem

} // end namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_IDGRIDPART_CAPABILITIES_HH
