/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_IDGRIDPART_INTERSECTIONITERATOR_HH
#define DUNE_FEM_GRIDPART_IDGRIDPART_INTERSECTIONITERATOR_HH

#include <dune/grid/common/intersectioniterator.hh>

#include <dune/fem/gridpart/idgridpart/intersection.hh>

namespace Dune
{

  namespace Fem
  {

    // IdIntersectionIterator
    // ----------------------

    template< class GridFamily >
    class IdIntersectionIterator
    {
      typedef IdIntersectionIterator< GridFamily > ThisType;

      typedef typename remove_const< GridFamily >::type::Traits Traits;

      typedef typename Traits::HostGridPartType::IntersectionIteratorType HostIntersectionIteratorType;

      typedef IdIntersection< const GridFamily > IntersectionImplType;

    public:
      typedef Dune::Intersection< const GridFamily, IdIntersection > Intersection;

      IdIntersectionIterator ( const HostIntersectionIteratorType &hostIterator )
      : intersection_( IntersectionImplType() ),
        hostIterator_( hostIterator )
      {}

      IdIntersectionIterator ( const ThisType &other )
      : intersection_( IntersectionImplType() ),
        hostIterator_( other.hostIterator_ )
      {}

      const ThisType &operator= ( const ThisType &other )
      {
        intersectionImpl() = IntersectionImplType();
        hostIterator_ = other.hostIterator_;
        return *this;
      }

      bool equals ( const ThisType &other ) const
      {
        return (hostIterator_ == other.hostIterator_);
      }
      
      void increment ()
      {
        ++hostIterator_;
        intersectionImpl() = IntersectionImplType();
      }

      const Intersection &dereference () const
      {
        if( !intersectionImpl() )
          intersectionImpl() = IntersectionImplType( *hostIterator_ );
        return intersection_;
      }

    private:
      IntersectionImplType &intersectionImpl () const
      {
        return intersection_.impl();
      }

      mutable Intersection intersection_;
      HostIntersectionIteratorType hostIterator_;
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_IDGRIDPART_INTERSECTIONITERATOR_HH
