/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#include <config.h>

#include <iostream>

#include <dune/fem/gridpart/leafgridpart.hh>
#include <dune/fem/operator/matrix/blockmatrix.hh>
#include <dune/fem/io/matlab/matlabstream.hh>

#include "testgrid.hh"

using namespace Dune;


int main()
{
  typedef Dune::GridSelector::GridType GridType;
  typedef LeafGridPart< GridType > GridPartType; 
  // test writing a vector and a matrix to 1 file
  Fem::MatlabOutStream s( "test.data" );
  
  Fem :: DynamicVector< double > v( 3 );
  for( unsigned int i = 0; i < 3; ++i )
    v[ i ] = i * (i+1);
  std :: cout << v << std :: endl;
  s << v;

  Fem::DenseMatrix<double> m(4,3);
  for( unsigned int i =0; i< 4; ++i )
  {
    for( unsigned int j = 0; j < 3; ++j )
      m[ i ][ j ]= j * (i+1);
  }
  m.print( std::cout );
  s << m;

  // test writing out a grid
  GridType &grid = TestGrid :: grid();
  GridPartType gridPart( grid );
  Fem::MatlabOutStream sg( "test.grid" );
  sg << gridPart;


  Fem::MatlabInStream in( "test.vector" );
  in >> v;

}
