/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_MISC_FUNCTOR_HH
#define DUNE_FEM_MISC_FUNCTOR_HH

#include <cstddef>

namespace Dune
{

  namespace Fem
  {

    // DefaultAssign
    // -------------

    struct DefaultAssign
    {
      template< class T >
      void operator() ( const T &a, T &b ) const
      {
        b = a;
      }
    };



    // AssignFunctor
    // -------------

    template< class Array, class Assign = DefaultAssign >
    struct AssignFunctor
    {
      explicit AssignFunctor ( Array &array, const Assign &assign = Assign() )
      : array_( array ),
        assign_( assign )
      {}

      template< class Element >
      void operator() ( const std::size_t i, const Element &e )
      {
        assign_( e, array_[ i ] );
      }

    private:
      Array &array_;
      Assign assign_;
    };



    // AssignSingleFunctor
    // -------------------

    template< class Value >
    struct AssignSingleFunctor
    {
      explicit AssignSingleFunctor ( const std::size_t i, Value &value )
      : i_( i ),
        value_( value )
      {}

      void operator() ( const std::size_t i, const Value &v )
      {
        if( i == i_ )
          value_ = v;
      }

    private:
      std::size_t i_;
      Value &value_;
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_MISC_FUNCTOR_HH
