/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
// Test from Chuck Allison in C/C++ Users Journal, September 2000:
// "The Simplest Automated Unit Test Framework That Could Possibly Work"
// Numeric test feature courtesy to P. Frauenfelder (Concepts)

#include "test.hh"
#include <cmath>
#include <iostream>
#include <typeinfo>     // Visual Studio requires /GR""
#include <limits>

#ifdef _MSC_VER
//Allow return-less mains:
#pragma warning(disable: 4541)
#endif

using namespace std;

namespace Dune {
const double Test::eps = std::numeric_limits<double>::epsilon();

void Test::doFloatTest(double arg1, double arg2, const std::string& lbl1,
                       const std::string& lbl2, const char* fname, 
                       long lineno, double relTol) {
  std::string lbl = lbl1 + std::string(" ~= ") + lbl2;
  double error;
  if (arg1 < 10.0*eps) {
    error = std::fabs(arg1-arg2);
  } else {
    error = std::fabs((arg1-arg2)/arg1);
  }
  if (error > relTol) {
    doFail(lbl, fname, lineno);
  } else {
    _succeed();
  }
}

void Test::doTest(bool cond, const std::string& lbl,
                  const char* fname, long lineno)
{
  if (!cond)
    doFail(lbl, fname, lineno);
  else
    _succeed();
}

void Test::doFail(const std::string& lbl,
                  const char* fname, long lineno)
{
  ++m_nFail;
  if (m_osptr)
    {
      *m_osptr << typeid(*this).name()
               << " failure: (" << lbl << "), "
               << fname
               << " (line " << lineno << ")\n";
    }
}

long Test::report() const
{
  if (m_osptr)
    {
      *m_osptr << "Test \"" 
               << typeid(*this).name() << "\":\n"
               << "\tPassed: " << m_nPass
               << "\tFailed: " << m_nFail
               << endl;
    }
  return m_nFail;
}

}
