/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#include <config.h>

#include <iostream>
#include <fstream>

#include <dune/common/exceptions.hh>
#include <dune/common/misc.hh>
#include <dune/common/stdstreams.cc>
#include <dune/common/timer.hh>

#include <dune/fem/misc/double.hh>
#include <dune/fem/misc/mpimanager.hh> 


using namespace Dune;

const int dim = GridSelector::dimworld;
const int ncomp = 1;

// include file with the description of the convection diffusion problem
#include "discretization.hh"

using namespace LDGExample;

template <typename Field, class GridImp, int dimR, int polOrd=0 >
struct DescriptionTraits
{
  enum { dim      = GridImp::dimension };
  enum { dimworld = GridImp::dimensionworld };

  enum { dimRange = dimR };

  typedef Field   FieldType;
  typedef GridImp GridType;

  // the model 
  typedef ModelParam  <FieldType,GridType,dimRange>  ModelParamType;
  typedef Model       <ModelParamType>  ModelType;

  // the discretisation 
  typedef DiscrParam  <ModelType,polOrd>  DiscrParamType;
};

int main (int argc, char **argv)
{
  // this method calls MPI_Init, if MPI is enabled
  MPIManager :: initialize(argc,argv);
  try 
  {
    // error message if called without parameter file
    if(argc < 2)
    {
      fprintf(stderr,"usage: %s <parameter file>\n",argv[0]);
    }
    
    // read data from the parameter file
    const char * paramname = "parameter";
    if(argc == 2) paramname = argv[1];

    std::string paramfile ( paramname );

    typedef DescriptionTraits <double,Dune::GridSelector::GridType,ncomp,2> DescrType;
    typedef DescrType :: ModelType ModelType;
    typedef DescrType :: DiscrParamType DiscrParamType;

    ModelType model(paramfile);

    Timer timer;
    simul<DiscrParamType>(model,paramfile);
    std::cout << "CPUtime: " << timer.elapsed() << " sec!\n";
  }
  catch (Dune::Exception& e)
  {
    std::cerr << e << std::endl;
    return 1;
  }
  return 0;
}

