/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_DGPASSSTUB_HH
#define DUNE_DGPASSSTUB_HH

#include <string>

#include <dune/fem/space/lagrangespace.hh>
#include <dune/fem/gridpart/leafgridpart.hh>
#include <dune/fem/function/adaptivefunction.hh>
#include <dune/fem/quadrature/elementquadrature.hh>

#include <dune/fem/pass/dgpass.hh>
#include <dune/fem/pass/dgdiscretemodel.hh>
#include <dune/fem/pass/selection.hh>

namespace Dune
{
  
  class DiscreteModelStub;

  struct DGStubTraits
  {
    enum { dim = GridSelector::GridType::dimension };
    typedef LeafGridPart< GridSelector::GridType > GridPartType;
    typedef FunctionSpace<double, double, dim , 1> FunctionSpaceType;
    typedef LagrangeDiscreteFunctionSpace<
      FunctionSpaceType, GridPartType, 1> DiscreteFunctionSpaceType;
    typedef DiscreteFunctionSpaceType SpaceType;
    typedef AdaptiveDiscreteFunction<DiscreteFunctionSpaceType> DestinationType;
    typedef ElementQuadrature< GridPartType, 0> VolumeQuadratureType;
    typedef ElementQuadrature< GridPartType, 1>  FaceQuadratureType;
    typedef FieldVector<double, dim > DomainType;
    typedef FieldVector<double, dim > FaceDomainType;
    typedef DiscreteFunctionSpaceType::RangeType RangeType;
    typedef DiscreteFunctionSpaceType::JacobianRangeType JacobianRangeType;

    typedef DiscreteModelStub DGDiscreteModelType;
  };
  
  class DiscreteModelStub
  : public DGDiscreteModelDefault< DGStubTraits >
  {
    typedef DGDiscreteModelDefault< DGStubTraits > BaseType;

  public:
    typedef Selector< 0 >::Base SelectorType;

    typedef DGStubTraits Traits;

    typedef Traits::FunctionSpaceType FunctionSpaceType;
    typedef Traits::GridPartType GridPartType;
    typedef GridPartType::GridType GridType;
    typedef Traits::DiscreteFunctionSpaceType DiscreteFunctionSpaceType;
    // * find common notation
    typedef Traits::SpaceType SpaceType;
    typedef Traits::DestinationType DestinationType;
    typedef Traits::VolumeQuadratureType VolumeQuadratureType;
    typedef Traits::FaceQuadratureType FaceQuadratureType;
    typedef Traits::DiscreteFunctionSpaceType::RangeType RangeType;
    typedef Traits::DiscreteFunctionSpaceType::JacobianRangeType JacobianRangeType;
    typedef GridPartType :: IntersectionIteratorType IntersectionIterator;
    typedef IntersectionIterator :: Intersection  Intersection;
    typedef GridType::Codim<0>::Entity EntityType;

    typedef BaseType :: MassFactorType MassFactorType; 
 
    typedef FieldVector<double, 3> DomainType;
  
  public:
    bool hasFlux() const
    {
      return true;
    }

    bool hasSource() const
    {
      return true;
    }

    bool hasMass () const
    {
      return true;
    }

    template <class ArgumentTuple, class FaceDomainType>
    double numericalFlux(const Intersection& it,
                         double time, const FaceDomainType& x,
                         const ArgumentTuple& uLeft, 
                         const ArgumentTuple& uRight,
                         RangeType& gLeft,
                         RangeType& gRight)
    { 
      std::cout << "numericalFlux()" << std::endl; 
      return 0.0;
    }

    template <class ArgumentTuple, class FaceDomainType>
    double boundaryFlux(const Intersection& it,
                        double time, const FaceDomainType& x,
                        const ArgumentTuple& uLeft,
                        RangeType& boundaryFlux)
    {
      std::cout << "boundaryFlux()" << std::endl;
      return 0.0;
    }

    template <class ArgumentTuple>
    void analyticalFlux(EntityType& en,
                        double time, const DomainType& x,
                        const ArgumentTuple& u, JacobianRangeType& f) 
    {
      std::cout << "analyticalFlux()" << std::endl;
    }

    template <class ArgumentTuple, class JacobianTuple>
    double source(const EntityType& en, 
                  const double time, 
                  const DomainType& x,
                  const ArgumentTuple& u, 
                  const JacobianTuple& jac, 
                  RangeType& s)
    {
      std::cout << "S()" << std::endl;
      return 0.0;
    }

    template <class ArgumentTuple>
    void mass(const EntityType& en,
              const double time,
              const DomainType& x,
              const ArgumentTuple& u,
              MassFactorType& m)
    {
      std::cout << "M()" << std::endl;
    }
  };

} // end namespace Dune

#endif
