/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_LAGRANGE_FIXTURE_HH
#define DUNE_LAGRANGE_FIXTURE_HH

//- Dune includes
#include <dune/fem/space/common/dofmanager.hh>
#include <dune/fem/space/lagrangespace.hh>
#include <dune/fem/gridpart/leafgridpart.hh>

namespace Dune {
  
  template <class GridImp, int polOrd>
  class Lagrange_Fixture {
  public:
    typedef GridImp GridType;
    typedef DofManager<GridType> DofManagerType;
    typedef LeafGridPart<GridType> GridPartType;
    typedef FunctionSpace<double, double, GridType :: dimension , 1> FunctionSpaceType;
    typedef LagrangeDiscreteFunctionSpace<
      FunctionSpaceType, GridPartType, polOrd> DiscreteFunctionSpaceType;
    
  public:
    Lagrange_Fixture(GridType& grid) :
      grid_(grid),
      dm_( DofManagerType::instance( grid_ ) ),
      part_(grid_),
      spc_(part_)
    {}

    GridType& grid() { return grid_; }
    DiscreteFunctionSpaceType& space() { return spc_; }
  private:
    GridType& grid_;
    DofManagerType& dm_;
    GridPartType part_;
    DiscreteFunctionSpaceType spc_;
  };
  
} // end namespace Dune
#endif
