/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_PASSSTUB_HH
#define DUNE_PASSSTUB_HH

#include <dune/fem/gridpart/leafgridpart.hh>
#include <dune/fem/space/lagrangespace.hh>
#include <dune/fem/function/adaptivefunction.hh>
#include <dune/fem/quadrature/elementquadrature.hh>

#include <dune/fem/pass/pass.hh>
#include <dune/fem/pass/dgdiscretemodel.hh>
#include <dune/fem/quadrature/elementquadrature.hh>
namespace Dune
{

  class ProblemStub;

  template< class GridImp >
  struct PassStubTraits
  {
    typedef GridImp GridType;
    typedef PassStubTraits<GridType> Traits;
    typedef ProblemStub DGDiscreteModelType;
    typedef FunctionSpace<double, double, GridType :: dimension, 1> FunctionSpaceType;
    typedef typename FunctionSpaceType :: RangeType  RangeType;
    typedef typename FunctionSpaceType :: DomainType DomainType;
    typedef typename FunctionSpaceType :: JacobianRangeType JacobianRangeType;
    typedef LeafGridPart<GridType> GridPartType;
    typedef ElementQuadrature<GridPartType,0> VolumeQuadratureType;
    typedef ElementQuadrature<GridPartType,1> FaceQuadratureType;
    typedef LagrangeDiscreteFunctionSpace<
      FunctionSpaceType, GridPartType, 1> DiscreteFunctionSpaceType;
    typedef AdaptiveDiscreteFunction<DiscreteFunctionSpaceType> DestinationType;
  };

  template< class PreviousPass, int pId = -1 >
  class PassStub
  : public Pass< PassStubTraits< GridSelector::GridType >, PreviousPass, pId >
  {
    typedef PassStub< PreviousPass, pId > ThisType;
    typedef Pass< PassStubTraits< GridSelector::GridType >, PreviousPass, pId > BaseType;

  public:
    typedef PassStubTraits< GridSelector::GridType > PassStubTraitsType;
    typedef PreviousPass PreviousPassType;

    typedef typename BaseType::TotalArgumentType ArgumentType;
    typedef typename PassStubTraitsType::DestinationType DestinationType;

  public:
    explicit PassStub( PreviousPassType &previousPass )
    : BaseType( previousPass )
    {}
      
    virtual void compute(const ArgumentType& arg, DestinationType& dest) const
    {}

    virtual void allocateLocalMemory() {}
  };


  struct ProblemStub
  : public DGDiscreteModelDefaultWithInsideOutside< PassStubTraits< GridSelector::GridType >, 0 > 
  {
    typedef PassStubTraits<GridSelector::GridType> Traits;
  };
}

#endif
