/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_ENTITYCOMMHELPER_HH
#define DUNE_FEM_ENTITYCOMMHELPER_HH

#include <dune/grid/common/gridenums.hh>

namespace Dune
{

namespace Fem
{

  template< InterfaceType iftype >
  struct EntityCommHelper;


  template<>
  struct EntityCommHelper< InteriorBorder_InteriorBorder_Interface >
  {
    static bool send ( const PartitionType p )
    {
      //return (p == InteriorEntity) || (p == BorderEntity);
      return (p == BorderEntity);
    }

    static bool receive ( const PartitionType p )
    {
      //return (p == InteriorEntity) || (p == BorderEntity);
      return (p == BorderEntity);
    }
  };


  template<>
  struct EntityCommHelper< InteriorBorder_All_Interface >
  {
    static bool send ( const PartitionType p )
    {
      return (p == InteriorEntity) || (p == BorderEntity);
    }

    static bool receive ( const PartitionType p )
    {
      //return true;
      return (p != InteriorEntity);
    }
  };


  template<>
  struct EntityCommHelper< Overlap_OverlapFront_Interface >
  {
    static bool send ( const PartitionType p )
    {
      //return (p == InteriorEntity) || (p == BorderEntity) || (p == OverlapEntity);
      return (p != FrontEntity) && (p != GhostEntity);
    }

    static bool receive ( const PartitionType p )
    {
      //return (p == InteriorEntity) || (p == BorderEntity) || (p == OverlapEntity) || (p == FrontEntity);
      return (p != GhostEntity);
    }
  };


  template<>
  struct EntityCommHelper< Overlap_All_Interface >
  {
    static bool send ( const PartitionType p )
    {
      //return (p == InteriorEntity) || (p == BorderEntity) || (p == OverlapEntity);
      return (p != FrontEntity) && (p != GhostEntity);
    }

    static bool receive ( const PartitionType p )
    {
      return true;
    }
  };


  template<>
  struct EntityCommHelper< All_All_Interface >
  {
    static bool send ( const PartitionType p )
    {
      return true;
    }

    static bool receive ( const PartitionType p )
    {
      return true;
    }
  };

} // end namespace Fem

} // end namespace Dune

#endif // #ifndef DUNE_FEM_ENTITYCOMMHELPER_HH
