/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_LOCALDGMASSMATRIX_HH
#define DUNE_FEM_LOCALDGMASSMATRIX_HH

#include <dune/fem/operator/1order/localmassmatrix.hh>

namespace Dune
{
  namespace Fem 
  {

  // External Forward Declarations
  // -----------------------------

  template< class FunctionSpaceImp, class GridPartImp, int polOrd, template< class > class BaseFunctionStorageImp >
  class DiscontinuousGalerkinSpace;

  template< class FunctionSpaceImp, class GridPartImp, int polOrd, template< class > class BaseFunctionStorageImp >
  class LegendreDiscontinuousGalerkinSpace; 



  // LocalMassMatrix
  // ---------------

  /** \brief Local Mass Matrix for DG space */
  template <class FunctionSpaceImp, class GridPartImp, int polOrd,
            template<class> class BaseFunctionStorageImp, 
            class VolumeQuadratureImp>
  class LocalMassMatrix< 
    DiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp >,
    VolumeQuadratureImp > 
    : public LocalMassMatrixImplementationDgOrthoNormal< 
        DiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp >, VolumeQuadratureImp >
  {
    typedef DiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp > DiscreteFunctionSpaceImp;
    typedef LocalMassMatrixImplementationDgOrthoNormal< DiscreteFunctionSpaceImp, VolumeQuadratureImp > BaseType;
  public:
    LocalMassMatrix( const DiscreteFunctionSpaceImp& spc, const int volQuadOrd = -1 )
      : BaseType( spc, volQuadOrd )
    {}
  };

  /** \brief Local Mass Matrix for Legendre space */
  template <class FunctionSpaceImp, class GridPartImp, int polOrd,
            template<class> class BaseFunctionStorageImp, 
            class VolumeQuadratureImp>
  class LocalMassMatrix< 
    LegendreDiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp >,
    VolumeQuadratureImp > 
    : public LocalMassMatrixImplementationDgOrthoNormal< 
        LegendreDiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp >, VolumeQuadratureImp >
  {
    typedef LegendreDiscontinuousGalerkinSpace< FunctionSpaceImp, GridPartImp, polOrd, BaseFunctionStorageImp > DiscreteFunctionSpaceImp;
    typedef LocalMassMatrixImplementationDgOrthoNormal< DiscreteFunctionSpaceImp, VolumeQuadratureImp > BaseType;
  public:
    LocalMassMatrix( const DiscreteFunctionSpaceImp& spc, const int volQuadOrd = -1 )
      : BaseType( spc, volQuadOrd )
    {}
  };

  } // end namespace Fem 

} // namespace Dune 

#endif // #ifndef DUNE_FEM_LOCALDGMASSMATRIX_HH
