/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_ORTHONORMALBASE_HH
#define DUNE_ORTHONORMALBASE_HH

namespace OrthonormalBase_1D 
{
  typedef const double* DomainType;
  typedef double* JacobianRangeType;

  double eval_line(const int baseNum,DomainType xi );
  void grad_line(const int baseNum, DomainType xi,
                 JacobianRangeType grad );

}
  
namespace OrthonormalBase_2D 
{
  typedef const double* DomainType;
  typedef double* JacobianRangeType;
  typedef double HessianRangeType[3];

  double eval_triangle_2d (const int baseNum, DomainType xi );
  double eval_quadrilateral_2d (const int baseNum, DomainType xi );
  void grad_triangle_2d (const int baseNum, DomainType xi,
                          JacobianRangeType grad );
  void hess_triangle_2d (const int baseNum, DomainType xi,
                         HessianRangeType &h );
  void grad_quadrilateral_2d (const int baseNum, DomainType xi,
                               JacobianRangeType grad );
}


namespace OrthonormalBase_3D 
{
  typedef const double* DomainType;
  typedef double* JacobianRangeType;

  double eval_tetrahedron_3d (const int baseNum, DomainType xi );
  double eval_pyramid_3d (const int baseNum, DomainType xi );
  double eval_prism_3d (const int baseNum, DomainType xi );
  double eval_hexahedron_3d (const int baseNum, DomainType xi );

  void grad_tetrahedron_3d (const int baseNum, DomainType xi,
                             JacobianRangeType grad );
  void grad_pyramid_3d (const int baseNum, DomainType xi,
                         JacobianRangeType grad );
  void grad_prism_3d (const int baseNum, DomainType xi,
                       JacobianRangeType grad );
  void grad_hexahedron_3d (const int baseNum, DomainType xi,
                            JacobianRangeType grad );
}
#endif
