/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_LAGRANGESPACE_BASEFUNCTIONS_HH
#define DUNE_LAGRANGESPACE_BASEFUNCTIONS_HH

#include "genericbasefunctions.hh"

namespace Dune
{

  namespace Fem 
  {

  // LagrangeBaseFunction
  // --------------------
   
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  class LagrangeBaseFunction
  : public BaseFunctionInterface< FunctionSpace >
  {
    typedef LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder > ThisType;
    typedef BaseFunctionInterface< FunctionSpace > BaseType;

  public:
    typedef typename GeometryWrapper< topologyId, dim >::GenericGeometryType
      GenericGeometryType;
    typedef GenericLagrangeBaseFunction< FunctionSpace, GenericGeometryType, pOrder >
      GenericBaseFunctionType;

    enum { numBaseFunctions = GenericBaseFunctionType::numBaseFunctions };

    typedef typename GenericBaseFunctionType :: DomainType DomainType;
    typedef typename GenericBaseFunctionType :: RangeType RangeType;

    LagrangeBaseFunction( unsigned int baseNum );

    virtual void evaluate ( const FieldVector< int, 0 > &diffVariable,
                            const DomainType &x,
                            RangeType &phi ) const;
    
    virtual void evaluate ( const FieldVector< int, 1 > &diffVariable,
                            const DomainType &x,
                            RangeType &phi ) const;

    virtual void evaluate ( const FieldVector< int, 2 > &diffVariable,
                            const DomainType &x,
                            RangeType &phi ) const;

    virtual int order () const;

  protected:
    GenericBaseFunctionType baseFunction_;
  };



  // LagrangeBaseFunctionFactory
  // ---------------------------

  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  class LagrangeBaseFunctionFactory
  : public BaseFunctionFactory< ScalarFunctionSpace >
  {
    typedef LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >
      ThisType;
    typedef BaseFunctionFactory< ScalarFunctionSpace > BaseType;

    template< class Topology >
    struct Switcher;

  public:
    using BaseType::geometry;

    explicit LagrangeBaseFunctionFactory ( GeometryType geometry );

    virtual ~LagrangeBaseFunctionFactory ();

    virtual BaseFunctionInterface< ScalarFunctionSpace > *
    baseFunction ( int i ) const;

    virtual int numBaseFunctions () const;
  };


  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 1, 1 >, 1, 1 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 2, 1 >, 2, 1 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 3, 1 >, 3, 1 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 1, 1 >, 1, 1 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 2, 1 >, 2, 1 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 3, 1 >, 3, 1 >;

  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 1, 1 >, 1, 2 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 2, 1 >, 2, 2 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 3, 1 >, 3, 2 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 1, 1 >, 1, 2 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 2, 1 >, 2, 2 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 3, 1 >, 3, 2 >;

  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 1, 1 >, 1, 3 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 2, 1 >, 2, 3 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< double, double, 3, 1 >, 3, 3 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 1, 1 >, 1, 3 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 2, 1 >, 2, 3 >;
  extern template class LagrangeBaseFunctionFactory< FunctionSpace< float, float, 3, 1 >, 3, 3 >;

  } // namespace Fem 

} // namespace Dune

#include "basefunctions_inline.hh"

#endif // #ifndef DUNE_LAGRANGESPACE_BASEFUNCTIONS_HH
