/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_LAGRANGESPACE_BASEFUNCTIONS_INLINE_HH
#define DUNE_LAGRANGESPACE_BASEFUNCTIONS_INLINE_HH

#include <dune/common/version.hh>
#if HAVE_DUNE_GEOMETRY
#include <dune/geometry/genericgeometry/topologytypes.hh>
#else
#include <dune/grid/genericgeometry/topologytypes.hh>
#endif
#include "basefunctions.hh"

namespace Dune
{

  namespace Fem 
  {

  // LagrangeBaseFunction
  // --------------------
  
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder >
    :: LagrangeBaseFunction ( unsigned int baseNum )
  : BaseType(),
    baseFunction_( baseNum )
  {}
  
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  void LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder >
    :: evaluate ( const FieldVector< int, 0 > &diffVariable,
                  const DomainType &x,
                  RangeType &phi ) const
  {
    return baseFunction_.evaluate( diffVariable, x, phi );
  }
  
  
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  void LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder >
    :: evaluate ( const FieldVector< int, 1 > &diffVariable,
                  const DomainType &x,
                  RangeType &phi ) const
  {
    return baseFunction_.evaluate( diffVariable, x, phi );
  }
  
  
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  void LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder >
    :: evaluate ( const FieldVector< int, 2 > &diffVariable,
                  const DomainType &x,
                  RangeType &phi ) const
  {
    return baseFunction_.evaluate( diffVariable, x, phi );
  }

  
  template< class FunctionSpace, unsigned int topologyId,
            unsigned int dim, unsigned int pOrder >
  int LagrangeBaseFunction< FunctionSpace, topologyId, dim, pOrder >
    :: order () const
  {
    return pOrder;
  }



  // LagrangeBaseFunctionFactory::TopologyId
  // ---------------------------------------

  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  template< class Topology >
  struct LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >::Switcher
  {
    typedef LagrangeBaseFunction< ScalarFunctionSpace, Topology::id, dim, pOrder > BaseFunction;

    static void apply( int &numBaseFunctions )
    {
      numBaseFunctions = BaseFunction::numBaseFunctions; 
    }

    static void apply ( const int &i, BaseFunctionInterface< ScalarFunctionSpace > *&baseFunction )
    {
      baseFunction = new BaseFunction( i );
    }
  };


  // LagrangeBaseFunctionFactory
  // ---------------------------

  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >
    ::LagrangeBaseFunctionFactory ( GeometryType geometry )
  : BaseType( geometry )
  {
    assert( geometry.dim() == dim );
  }


  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >
    :: ~LagrangeBaseFunctionFactory ()
  {}


  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  BaseFunctionInterface< ScalarFunctionSpace > *
  LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >::baseFunction ( int i ) const
  {
    BaseFunctionInterface< ScalarFunctionSpace > *baseFunction;
    const unsigned int topologyId = geometry().id();
    GenericGeometry::IfTopology< Switcher, dim >::apply( topologyId, i, baseFunction );
    return baseFunction;
  }
 

  template< class ScalarFunctionSpace, unsigned int dim, unsigned int pOrder >
  int
  LagrangeBaseFunctionFactory< ScalarFunctionSpace, dim, pOrder >::numBaseFunctions () const
  {
    int numBaseFunctions;
    const unsigned int topologyId = geometry().id();
    GenericGeometry::IfTopology< Switcher, dim >::apply( topologyId, numBaseFunctions );
    return numBaseFunctions;
  }

  } // namespace Fem 

} // namespace Dune 

#endif // #ifndef DUNE_LAGRANGESPACE_BASEFUNCTIONS_INLINE_HH
