/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2012 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2012 Andreas Dedner
  Copyright (C) 2005 - 2010 Mirko Kraenkel
  Copyright (C) 2006 - 2012 Martin Nolte
  Copyright (C) 2011 - 2012 Tobias Malkmus

  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_REDUCEDBASISSPACE_COMMHANDLE_HH
#define DUNE_FEM_REDUCEDBASISSPACE_COMMHANDLE_HH

#include <dune/grid/common/datahandleif.hh>

#include <dune/fem/space/common/commoperations.hh>

namespace Dune
{
  namespace Fem
  {

  template< class DiscreteFunction, class Operation >
  class ReducedBasisCommDataHandle
  : public CommDataHandleIF
    < ReducedBasisCommDataHandle< DiscreteFunction, Operation >,
      typename DiscreteFunction :: RangeFieldType >
  {
  public:
    typedef DiscreteFunction DiscreteFunctionType;

    typedef Operation OperationType;

    typedef typename DiscreteFunctionType :: RangeFieldType DataType;

  private:
    typedef ReducedBasisCommDataHandle< DiscreteFunctionType, OperationType >
      ThisType;
    typedef CommDataHandleIF< ThisType, DataType > BaseType;

  public:
    typedef typename DiscreteFunctionType :: DiscreteFunctionSpaceType
      DiscreteFunctionSpaceType;

  protected:
    DiscreteFunctionType *const function_;

  public:
    inline explicit
    ReducedBasisCommDataHandle ( DiscreteFunctionType &function )
    : function_( &function )
    {}

    inline
    ReducedBasisCommDataHandle ( const ThisType &other )
    : function_( other.function_ )
    {}

  private:
    ThisType &operator= ( const ThisType & );

  public:
    inline bool contains ( int dim, int codim ) const
    {
      return false;
    }

    inline bool fixedsize ( int dim, int codim ) const
    {
      return true;
    }

    template< class MessageBuffer, class Entity >
    void gather ( MessageBuffer &buffer, const Entity &entity ) const
    {
    }

    template< class MessageBuffer, class Entity >
    void scatter ( MessageBuffer &buffer, const Entity &entity, size_t n )
    {
    }
    
    template< class Entity >
    size_t size ( const Entity &entity ) const
    {
      return 0;
    }
  };

  } // namespace Fem
  
} // namespace Dune 

#endif
